dojo.provide("dijit.form._Spinner");

dojo.require("dijit.form.ValidationTextbox");
dojo.require("dijit.util.typematic");
dojo.require("dijit.util.sniff");

dojo.declare(
	"dijit.form._Spinner",
	dijit.form.RangeBoundTextbox,
	{
		// summary: Mixin for validation widgets with a spinner
		// description: This class basically (conceptually) extends dijit.form.ValidationTextbox.
		//	It modifies the template to have up/down arrows, and provides related handling code.

		// defaultTimeout: Number
		//      number of milliseconds before a held key or button becomes typematic
		defaultTimeout: 500,

		// timeoutChangeRate: Number
		//      fraction of time used to change the typematic timer between events
		//      1.0 means that each typematic event fires at defaultTimeout intervals
		//      < 1.0 means that each typematic event fires at an increasing faster rate
		timeoutChangeRate: 0.90,

		// smallDelta: Number
		//      adjust the value by this much when spinning using the arrow keys/buttons
		smallDelta: 1,
		// largeDelta: Number
		//      adjust the value by this much when spinning using the PgUp/Dn keys
		largeDelta: 10,

		templateString:"<fieldset class=\"dojoSpinner dojoInputField\" dojoAttachPoint='nodeWithBorder' id='${this.id}'\n\t><input dojoAttachPoint='textbox' type='${this.type}' dojoAttachEvent='onblur;onfocus;onkeyup;'\n\t\tvalue='${this.value}' name='${this.name}' size='${this.size}' maxlength='${this.maxlength}'\n\t\twaiRole=\"spinbutton\" autocomplete=\"off\" tabIndex='${this.tabIndex}'\n\t><fieldset class=\"dojoSpinnerArrowGroup\"\n                ><fieldset dojoAttachPoint=\"upArrowNode\" class=\"dojoSpinnerButton\"\n                        ><div class=\"dojoSpinnerArrow\"\n                                ><div class=\"dojoSpinnerUpArrow\"></div\n                        ></div\n                ></fieldset\n                ><fieldset dojoAttachPoint=\"downArrowNode\" class=\"dojoSpinnerButton\"\n                        ><div class=\"dojoSpinnerArrow\"\n                                ><div class=\"dojoSpinnerDownArrow\"></div\n                        ></div\n                ></fieldset\n\t></fieldset\n\t><span dojoAttachPoint='messageSpan' class='dojoValidationTextboxMessage' waiRole='alert'></span\n></fieldset>\n",

		adjust: function(/* Object */ val, /*Number*/ delta){
			// summary: user replaceable function used to adjust a primitive value(Number/Date/...) by the delta amount specified
			// the val is adjusted in a way that makes sense to the object type
			return val;
		},

		_arrowPressed: function(/*Node*/ nodePressed, /*Number*/ direction){
			dojo.addClass(nodePressed, "dojoSpinnerButtonPushed");
			this.setValue(this.adjust(this.getValue(), direction*this.smallDelta));
		},

		_arrowReleased: function(/*Node*/ node){
			this._wheelTimer = null;
			this.textbox.focus();
			dojo.removeClass(node, "dojoSpinnerButtonPushed");
		},

		_typematicCallback: function(/*Number*/ count, /*DOMNode*/ node, /*Event*/ evt){
			if(node == this.textbox){ node = (evt.keyCode == dojo.keys.UP_ARROW) ? this.upArrowNode : this.downArrowNode; }
			if(count == -1){ this._arrowReleased(node); }
			else{ this._arrowPressed(node, (node == this.upArrowNode) ? 1 : -1); }
		},

		_wheelTimer: null,
		_mouseWheeled: function(/*Event*/ evt){
			dojo.stopEvent(evt);
			var scrollAmount = 0;
			if(typeof evt.wheelDelta == 'number'){ // IE
				scrollAmount = evt.wheelDelta;
			}else if (typeof evt.detail == 'number'){ // Mozilla+Firefox
				scrollAmount = -evt.detail;
			}
			if(scrollAmount > 0){
				var node = this.upArrowNode;
				var dir = +1;
			}else if (scrollAmount < 0){
				var node = this.downArrowNode;
				var dir = -1;
			}else{ return; }
			this._arrowPressed(node, dir);
			if (this._wheelTimer != null){
				clearTimeout(this._wheelTimer);
			}
			var _this = this;
			this._wheelTimer = setTimeout(function(){_this._arrowReleased(node);}, 50);
		},

		postCreate: function(){
			dijit.form._Spinner.superclass.postCreate.apply(this, arguments);

			// textbox and domNode get the same style but the css separates the 2 using !important
			if(this.srcNodeRef){
				dojo.style(this.textbox, "cssText", this.srcNodeRef.style.cssText);
				this.textbox.className += " " + this.srcNodeRef.className;
			}

			// extra listeners
			this.connect(dojo.isIE ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
			dijit.util.typematic.addListener(this.upArrowNode, this.textbox, {keyCode:dojo.keys.UP_ARROW,ctrlKey:false,altKey:false,shiftKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout);
			dijit.util.typematic.addListener(this.downArrowNode, this.textbox, {keyCode:dojo.keys.DOWN_ARROW,ctrlKey:false,altKey:false,shiftKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout);
		}
});
